package org.artifactory.ph;

import org.apache.http.HttpStatus;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.properties.PropertiesService;
import org.artifactory.api.request.DownloadService;
import org.artifactory.api.request.UploadService;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.api.security.SecurityService;
import org.artifactory.aql.AqlService;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.common.repo.PackageType;
import org.artifactory.common.repo.RepoInfo;
import org.artifactory.common.resource.BaseResource;
import org.artifactory.confprop.ConfProps;
import org.artifactory.core.api.ph.exception.PackageVirtualTraversalException;
import org.artifactory.core.api.ph.listener.PackageArtifactEventListener;
import org.artifactory.core.api.ph.service.PackageArtifactService;
import org.artifactory.core.api.ph.service.PackageArtifactoryConfigService;
import org.artifactory.core.api.ph.service.PackageRepoIndexCalculatorService;
import org.artifactory.core.api.ph.service.PackageRepoService;
import org.artifactory.core.api.ph.service.PackageSearchArtifactService;
import org.artifactory.core.api.ph.service.PackageSecurityService;
import org.artifactory.core.api.ph.service.PackageSpecificationAdapter;
import org.artifactory.core.api.ph.service.PackageUploadService;
import org.artifactory.core.api.ph.virtual.resolution.PackageMetadataRow;
import org.artifactory.core.api.ph.virtual.resolution.PackageMetadataVersionsSet;
import org.artifactory.core.api.ph.virtual.resolution.PackageRepo;
import org.artifactory.core.api.ph.virtual.resolution.PackageRequestContext;
import org.artifactory.core.api.ph.virtual.resolution.PackageVirtualResolutionAdapter;
import org.artifactory.repo.interceptor.storage.StorageInterceptorAdapter;
import org.artifactory.repo.service.InternalRepositoryService;
import org.artifactory.spring.ConditionalOnMissingBean;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;

import javax.annotation.Priority;
import java.io.InputStream;
import java.util.LinkedHashSet;
import java.util.Map;

/**
 * Beans configuration class
 */
@SuppressWarnings("SpringJavaInjectionPointsAutowiringInspection")
@Configuration
public class ArtifactoryPackageBeansConfig {

    /**
     * Expose {@link org.artifactory.common.property.ArtifactorySystemProperties} as spring bean type of
     * {@link ConfProps}
     */
    @Bean
    public ConfProps getConfProps() {
        return ArtifactoryHome.get().getArtifactoryProperties();
    }


    @Bean
    public PackageRepoService packageRepoService(InternalRepositoryService repositoryService,
            PropertiesService propertiesService) {
        return new PackageRepoServiceImpl(repositoryService, propertiesService);
    }

    @Bean
    public PackageArtifactService packageDownloadService(DownloadService downloadService,
            InternalRepositoryService repositoryService) {
        return new PackageArtifactServiceImpl(downloadService, repositoryService);
    }

    @Bean
    public PackageUploadService packageUploadService(InternalRepositoryService repositoryService,
            UploadService uploadService) {
        return new PackageUploadServiceImpl(repositoryService, uploadService);
    }

    @Bean
    public StorageInterceptorAdapter packageStorageInterceptorAdapter(PackageArtifactEventListener packagesListener,
            InternalRepositoryService repositoryService, PackageSpecificationAdapter packageSpecificationAdapter) {
        return new PackageStorageInterceptorImpl(packagesListener, repositoryService, packageSpecificationAdapter);
    }

    @Bean
    public PackageSecurityService packageSecurityService(AuthorizationService authorizationService,
            SecurityService securityService) {
        return new PackageSecurityServiceImpl(authorizationService, securityService);
    }

    @Bean
    public PackageSearchArtifactService packageSearchArtifactService(AqlService aqlService,
            AuthorizationService authorizationService) {
        return new PackageSearchArtifactServiceImpl(aqlService, authorizationService);
    }

    @Bean
    public PackageSpecification packageSpecification(PackageSpecificationAdapter packageSpecificationAdapter) {
        return new PackageSpecificationImpl(packageSpecificationAdapter);
    }

    @Bean
    public PackageArtifactoryConfigService packageRepoDescriptorService(
            InternalRepositoryService internalRepositoryService, CentralConfigService centralConfigService) {
        return new PackageArtifactoryConfigServiceImpl(internalRepositoryService, centralConfigService);
    }

    @Bean
    public PackageVirtualResolutionService<PackageMetadataRow> packageVirtualResolutionService(
            PackageVirtualResolutionAdapter<PackageMetadataRow> packageVirtualResolutionAdapter) {
        return new PackageVirtualResolutionServiceImpl(packageVirtualResolutionAdapter);
    }


    /**
     * Configuration class for default beans that should be loaded only in None PRO run
     * Must have the lowest priority since has condition on missing bean.
     * We want to wait for the real beans a chance to be loaded, and if not, we load the default beans.
     */
    @Configuration
    @Priority(Ordered.LOWEST_PRECEDENCE)
    public static class PackageOssConfig {

        /**
         * Note: The method name must be the same as the method name of the real impl.
         * See
         * (com.jfrog.ph.service.InternalPackageHandlerService)}
         */
        @Bean
        @ConditionalOnMissingBean(PackageArtifactEventListener.class)
        public PackageArtifactEventListener packageArtifactEventListener() {
            return new PackageArtifactEventListener() {
            };
        }

        /**
         * Note: The method name must be the same as the method name of the real impl.
         * See
         * (com.jfrog.ph.service.InternalPackageHandlerService,
         * org.artifactory.core.api.ph.service.PackageRepoService,
         * org.artifactory.core.api.ph.service.PackageDownloadService,
         * org.artifactory.core.api.ph.service.PackageUploadService,
         * com.jfrog.ph.service.PackageHandlerConflictsGuardService,
         * org.artifactory.confprop.ConfProps)}
         */
        @Bean
        @ConditionalOnMissingBean(PackageVirtualResolutionAdapter.class)
        public PackageVirtualResolutionAdapter<PackageMetadataRow> packageVirtualAdapter() {
            return new PackageVirtualResolutionAdapter<>() {
                @Override
                public boolean shouldCacheMergedMetadata(String virtualRepoKey, String path) {
                    return false;
                }

                @Override
                public boolean shouldUsePackageVirtualResolution(String virtualRepoKey) {
                    return false;
                }

                @Override
                public BaseResource getItemInfoFromVirtual(PackageRepo virtualRepo,
                        PackageRequestContext context) {
                    return null;
                }

                @NotNull
                @Override
                public Map<String, LinkedHashSet<PackageMetadataRow>> extract(BaseResource res, String virtualRepoKey,
                        String originalPath)
                        throws PackageVirtualTraversalException {
                    String message = "The 'extract' method is not supported in OSS release";
                    throw new PackageVirtualTraversalException(HttpStatus.SC_INTERNAL_SERVER_ERROR,
                            "Unsupported request", message);
                }

                @NotNull
                @Override
                public InputStream toInputStream(
                        Map<String, PackageMetadataVersionsSet<PackageMetadataRow>> metadata,
                        PackageRequestContext requestContext) throws PackageVirtualTraversalException {
                    String message = "The 'toInputStream' method is not supported in OSS release";
                    throw new PackageVirtualTraversalException(HttpStatus.SC_INTERNAL_SERVER_ERROR,
                            "Unsupported request", message);
                }
            };
        }

        /**
         * Note: The method name must be the same as the method name of the real impl.
         * See
         * (com.jfrog.ph.service.InternalPackageHandlerService)}
         */
        @Bean
        @ConditionalOnMissingBean(PackageRepoIndexCalculatorService.class)
        public PackageRepoIndexCalculatorService packageRepoIndexCalculatorService() {
            return (packageType, repoKey) -> {
            };
        }

        /**
         * Note: The method name must be the same as the method name of the real impl.
         * See
         * (com.jfrog.ph.service.InternalPackageHandlerService)}
         */
        @Bean
        @ConditionalOnMissingBean(PackageSpecificationAdapter.class)
        public PackageSpecificationAdapter packageSpecificationAdapter() {
            return new PackageSpecificationAdapter() {

                @Override
                public boolean isPackageArtifact(@NotNull String repoKey, @NotNull String path) {
                    return false;
                }

                @Override
                public boolean isExpirable(@NotNull PackageType packageType, @NotNull String path) {
                    return false;
                }

                @Override
                public boolean isLocalGenerated(@NotNull PackageType packageType, @NotNull String path) {
                    return false;
                }

                @Override
                public boolean isLocallyGeneratedProperty(@NotNull PackageType packageType,
                        @NotNull String propertyKey) {
                    return false;
                }

                @Override
                public boolean isSupportedPackage(PackageType packageType) {
                    return false;
                }

                @Override
                public boolean isSupportedPackage(String repoKey) {
                    return false;
                }

                @Override
                public boolean hasReindex(RepoInfo repoInfo) {
                    return false;
                }
            };
        }
    }
}
