package org.artifactory.ph;

import com.fasterxml.jackson.core.type.TypeReference;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.artifactory.api.events.EventLogHintManager;
import org.artifactory.api.repo.exception.RepoRejectException;
import org.artifactory.api.rest.restmodel.JsonUtil;
import org.artifactory.common.MutableStatusHolder;
import org.artifactory.common.repo.PackageType;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.listener.PackageArtifactEventListener;
import org.artifactory.core.api.ph.listener.PackageEventListenerContext;
import org.artifactory.core.api.ph.model.PackageArtifact;
import org.artifactory.core.api.ph.service.PackageSpecificationAdapter;
import org.artifactory.event.EventHintType;
import org.artifactory.fs.ItemInfo;
import org.artifactory.fs.RepoResource;
import org.artifactory.md.Properties;
import org.artifactory.repo.Repo;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.interceptor.storage.StorageInterceptorAdapter;
import org.artifactory.repo.remote.interceptor.RemoteRepoInterceptor;
import org.artifactory.repo.service.InternalRepositoryService;
import org.artifactory.sapi.fs.VfsItem;
import org.artifactory.sapi.interceptor.context.DeleteContext;
import org.artifactory.sapi.interceptor.context.InterceptorCreateContext;
import org.artifactory.sapi.interceptor.context.InterceptorMoveCopyContext;
import org.artifactory.util.StringInputStream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nullable;
import java.util.HashMap;
import java.util.Map;

public class PackageStorageInterceptorImpl extends StorageInterceptorAdapter implements RemoteRepoInterceptor {
    private static final Logger log = LoggerFactory.getLogger(PackageStorageInterceptorImpl.class);
    private final PackageSpecificationAdapter packageSpecificationAdapter;
    private final PackageArtifactEventListener packagesListener;
    private final InternalRepositoryService repositoryService;

    public PackageStorageInterceptorImpl(PackageArtifactEventListener packagesListener,
            InternalRepositoryService repositoryService, PackageSpecificationAdapter packageSpecificationAdapter) {
        this.packagesListener = packagesListener;
        this.repositoryService = repositoryService;
        this.packageSpecificationAdapter = packageSpecificationAdapter;
    }

    @Override
    public void afterCreate(VfsItem<? extends ItemInfo> fsItem, MutableStatusHolder statusHolder,
            InterceptorCreateContext ctx) {
        if (fsItem.isFolder()) {
            return;
        }

        try {
            if (packageSpecificationAdapter.isSupportedPackage(fsItem.getRepoKey())) {
                PackageArtifact artifact = getArtifact(fsItem);
                if (artifact != null) {
                    Repo<?> repo = repositoryService.repositoryByKey(artifact.getRepoKey());
                    if (repo != null && repo.isLocal() && !repo.isCache()) {
                        packagesListener.afterCreate(artifact);
                    }
                }
            }
        } catch (Exception e) {
            error(e, "afterCreate", fsItem.getRepoPath());
        }
    }

    @Override
    public void beforeDelete(VfsItem<? extends ItemInfo> fsItem, MutableStatusHolder statusHolder, DeleteContext ctx) {
        if (fsItem.isFolder() || ctx.isTriggeredByMove()) {
            return;
        }
        try {
            if (packageSpecificationAdapter.isSupportedPackage(fsItem.getRepoKey())) {
                PackageArtifact artifact = getArtifact(fsItem);
                PackageEventListenerContext context = new PackageEventListenerContext();
                if (artifact != null) {
                    Repo<?> repo = repositoryService.repositoryByKey(artifact.getRepoKey());
                    if (repo != null && repo.isLocal() && !repo.isCache()) {
                        packagesListener.beforeDelete(artifact, context);
                        storeHints(artifact, context);
                    }
                }
            }
        } catch (Exception e) {
            error(e, "beforeDelete", fsItem.getRepoPath());
        }
    }

    @Override
    public void afterDelete(VfsItem<? extends ItemInfo> fsItem, MutableStatusHolder statusHolder, DeleteContext ctx) {
        if (fsItem.isFolder() || ctx.isTriggeredByMove()) {
            return;
        }
        try {
            if (packageSpecificationAdapter.isSupportedPackage(fsItem.getRepoKey())) {
                PackageArtifact artifact = getArtifact(fsItem);
                if (artifact != null) {
                    Repo<?> repo = repositoryService.repositoryByKey(artifact.getRepoKey());
                    if (repo != null && repo.isLocal() && !repo.isCache()) {
                        packagesListener.afterDelete(artifact);
                    }
                }
            }
        } catch (Exception e) {
            error(e, "afterDelete", fsItem.getRepoPath());
        }
    }

    @Override
    public void afterMove(VfsItem<? extends ItemInfo> sourceItem, VfsItem<? extends ItemInfo> targetItem,
            MutableStatusHolder statusHolder, Properties properties, InterceptorMoveCopyContext ctx) {
        if (sourceItem == null || targetItem == null || sourceItem.isFolder()) {
            return;
        }
        try {
            Repo<?> repo = repositoryService.repositoryByKey(sourceItem.getRepoKey());
            if (repo != null && repo.isLocal() && !repo.isCache() &&
                    packageSpecificationAdapter.isSupportedPackage(sourceItem.getRepoKey())) {
                PackageArtifact source = getArtifact(sourceItem);
                PackageArtifact target = getArtifact(targetItem);
                packagesListener.afterMove(source, target);
            }
        } catch (Exception e) {
            error(e, "afterMove", sourceItem.getRepoPath());
        }
    }

    @Override
    public void afterCopy(VfsItem<? extends ItemInfo> sourceItem, VfsItem<? extends ItemInfo> targetItem,
            MutableStatusHolder statusHolder, Properties properties, InterceptorMoveCopyContext ctx) {
        if (sourceItem == null || targetItem == null || sourceItem.isFolder()) {
            return;
        }
        try {
            Repo<?> repo = repositoryService.repositoryByKey(sourceItem.getRepoKey());
            if (repo != null && repo.isLocal() && !repo.isCache() &&
                    packageSpecificationAdapter.isSupportedPackage(sourceItem.getRepoKey())) {
                PackageArtifact target = getArtifact(targetItem);
                PackageArtifact source = getArtifact(sourceItem);
                packagesListener.afterCopy(source, target);
            }
        } catch (Exception e) {
            error(e, "afterCopy", sourceItem.getRepoPath());
        }
    }

    @Override
    public void afterPropertyCreate(VfsItem<? extends ItemInfo> fsItem, MutableStatusHolder statusHolder, String key,
            String... values) {
        if (fsItem.isFolder()) {
            return;
        }
        try {
            if (packageSpecificationAdapter.isSupportedPackage(fsItem.getRepoKey())) {
                PackageArtifact artifact = getArtifact(fsItem);
                if (artifact != null) {
                    Repo<?> repo = repositoryService.repositoryByKey(artifact.getRepoKey());
                    if (repo != null && repo.isLocal() && !repo.isCache()) {
                        packagesListener.afterPropertyChange(artifact, key, false, values);
                    }
                }
            }
        } catch (Exception e) {
            error(e, "afterPropertyCreate", fsItem.getRepoPath());
        }
    }

    @Override
    public void afterPropertyDelete(VfsItem<? extends ItemInfo> fsItem, MutableStatusHolder statusHolder, String key) {
        try {
            if (fsItem.isFolder()) {
                return;
            }
            if (packageSpecificationAdapter.isSupportedPackage(fsItem.getRepoKey())) {
                PackageArtifact artifact = getArtifact(fsItem);
                if (artifact != null) {
                    Repo<?> repo = repositoryService.repositoryByKey(artifact.getRepoKey());
                    if (repo != null && repo.isLocal() && !repo.isCache()) {
                        packagesListener.afterPropertyChange(artifact, key, true);
                    }
                }
            }
        } catch (Exception e) {
            error(e, "afterPropertyDelete", fsItem.getRepoPath());
        }
    }

    @Override
    public void afterRemoteDownload(RepoResource remoteResource) throws RepoRejectException {
        try {
            if (packageSpecificationAdapter.isSupportedPackage(remoteResource.getRepoKey())) {
                PackageArtifact artifact = getArtifact(remoteResource);
                if (artifact != null && repositoryService.remoteRepositoryByKey(artifact.getRepoKey()) != null) {
                    packagesListener.afterRemoteDownload(artifact);
                }
            }
        } catch (Exception e) {
            error(e, "afterRemoteDownload", remoteResource.getRepoPath());
        }
    }

    @Nullable
    private PackageArtifact getArtifact(VfsItem<? extends ItemInfo> fsItem) {
        Repo<?> repo = getRepo(fsItem.getRepoKey());
        if (repo == null) {
            return null;
        }
        PackageType packageType = packageType(repo);
        if (packageType == null) {
            return null;
        }
        return PackageArtifactMapper.artifact(fsItem, packageType);
    }

    @Nullable
    private PackageArtifact getArtifact(RepoResource remoteResource) {
        Repo<?> repo = getRepo(remoteResource.getRepoKey());
        if (repo == null) {
            return null;
        }
        PackageType packageType = packageType(repo);
        if (packageType == null) {
            return null;
        }
        return PackageArtifactMapper.artifact(remoteResource, packageType);
    }

    @Nullable
    private Repo<?> getRepo(String repoKey) {
        Repo<?> repo = repositoryService.repositoryByKey(repoKey);
        if (repo == null) {
            log.error("Failed to identify repo for key: {} ", repoKey);
        }
        return repo;
    }

    private void storeHints(PackageArtifact artifact, PackageEventListenerContext context) {
        if (MapUtils.isNotEmpty(context.getEventExtraInfo())) {
            //If we already have hint value for this hint, we should load it and add the new hints
            Map<String, Object> extraInfo = null;
            if (EventLogHintManager.contains(EventHintType.PackageIndexExtraInfo)) {
                String hintValue =
                        EventHintType.getValue(EventLogHintManager.getHints(), EventHintType.PackageIndexExtraInfo);
                if (StringUtils.isNotEmpty(hintValue)) {
                    try {
                        extraInfo = JsonUtil.mapDataToModel(new StringInputStream(hintValue), new TypeReference<>() {
                        });
                    } catch (Exception e) {
                        log.error("Failed to deserialize hint: {}, value: {}",
                                EventHintType.PackageIndexExtraInfo.name(), hintValue, e);
                    }
                }
            }

            if (extraInfo == null) {
                extraInfo = new HashMap<>(context.getEventExtraInfo().size());
            }

            extraInfo.putAll(context.getEventExtraInfo());
            String hintValue = JsonUtil.jsonToString(extraInfo);
            EventLogHintManager.put(EventHintType.PackageIndexExtraInfo, hintValue);
            log.debug("Event hint {} added on {}/{}: {}", EventHintType.PackageIndexExtraInfo.name(),
                    artifact.getRepoKey(), artifact.getPath(), hintValue);

        }
    }

    private PackageType packageType(@Nullable Repo<?> repo) {
        try {
            if (repo == null || repo.getDescriptor() == null) {
                return null;
            }
            return PackageType.valueOf(repo.getDescriptor().getType().name().toUpperCase());
        } catch (IllegalArgumentException e) {
            return null;
        }
    }

    private void error(Exception e, String type, RepoPath repoPath) {
        if (e instanceof PackageException) {
            PackageException pe = (PackageException) e;
            log.error("Failed to handle {} event on {}: {}, {}, {}, {}", type, repoPath.toPath(),
                    pe.getStatus(), pe.getInternalReason(), pe.getPublicReason(), e.getMessage());
        } else {
            log.error("Failed to handle {} event on {}: {}", type, repoPath.toPath(), e.getMessage());
        }
        log.debug("", e);
    }
}
