package org.artifactory.core.api.ph.service;

import com.google.common.collect.Multimap;
import org.artifactory.common.repo.PackageType;
import org.artifactory.common.repo.RepoInfo;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.virtual.resolution.PackageRepo;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;

public interface PackageRepoService {

    /**
     * checks if repository is virtual
     *
     * @param repoKey repository to check
     * @return true the repository is virtual repository
     */
    boolean isVirtual(String repoKey);

    /**
     * checks if repository is remote
     *
     * @param repoKey repository to check
     * @return true the repository is remote repository
     */
    boolean isRemote(String repoKey);

    /**
     * checks if repository is local
     *
     * @param repoKey repository to check
     * @return true the repository is local repository
     */
    boolean isLocal(String repoKey);

    /**
     * Moves repository path (pointing to a folder) to another absolute target. The move will only move paths the user
     * has permissions to move and paths that are accepted by the target repository.
     *
     * @param repoKey    The non-cached local repository name.
     * @param targetPath The target path to move the path to.
     * @throws PackageException in that the source path doesn't exist or internal error
     */
    void move(String repoKey, String sourcePath, String targetPath) throws PackageException;

    /**
     * Resolve the package type from repo key
     *
     * @param repoKey the repository to check
     * @return the above repository package type
     */
    @Nullable
    PackageType getPackageType(String repoKey);

    /**
     * Resolve the repo info package type (cran, maven, debian...) and repo type(local, virtual and remote)
     *
     * @param repoKey the repository to check
     * @return RepoInfo which is pair of package type and repo type
     * @throws PackageException in any case of not found or internal error
     */
    @NotNull
    RepoInfo getRepoInfo(String repoKey) throws PackageException;

    /**
     * Set all attribute key values on the given path.
     * Existing prop-keys on the path will get overwritten.
     *
     * @param path       The path for which to set the attribute values
     * @param properties Multimap for keys with multiple values
     */
    void setAttributes(String repoKey, String path, Multimap<String, String> properties);

    /**
     * Checks if an artifact exists locally
     *
     * @param repoKey the artifact repo
     * @param path    the artifact path
     * @return true if artifact exist
     */
    boolean existLocally(String repoKey, String path);

    /**
     * Undeploy an artifact
     *
     * @param repoKey the artifact repository key
     * @param path    the artifact path
     */
    void undeployArtifact(String repoKey, String path);

    /**
     * Return virtual repository sub repositories, the repositories are sorted according to the virtual repository logic
     * Which is prioritized (local->remote->virtual) prioritized (local->remote->virtual)
     *
     * @param repoKey the virtual repository to resolve from.
     * @return List of sorted sub repositories
     * @throws PackageException in any case of not found or internal error
     */
    List<? extends PackageRepo> getSubRepositories(String repoKey) throws PackageException;

    /**
     * In case of a Virtual repoKey, return the default deployment local repoKey - if there is.
     * In case of an exist local repoKey, return the local repoKey.
     * Any other case, will get an ExposedException.
     *
     * @return A local repoKey.
     * @throws PackageException in any case of not found or internal error
     */
    String getDefaultDeploymentRepoKey(String repoKey) throws PackageException;

    /**
     * This function cleaning the given  cache path of a virtual repository related to the given aggregated local
     * repository.
     * Use this function as a system
     *
     * @param localRepoKey            - repokey of the local repository
     * @param virtualRepoMetadataRoot - cache path to clean
     */
    void undeployVirtualMetadataCache(String localRepoKey, String virtualRepoMetadataRoot);
}