package org.artifactory.lifecycle.storage.db.init;

import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.converter.VersionProviderImpl;
import org.artifactory.version.ArtifactoryServiceVersionProvider;
import org.artifactory.version.CompoundVersionDetails;
import org.jfrog.common.ClockUtils;
import org.jfrog.config.ifc.ConfigurationManager;
import org.testng.annotations.Test;

import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

@Test
public class PostDbSchemaInitializationManagerTest {

    @Test
    public void testAcquireInitLockInHaAndVersionIsNotCurrent() {
        VersionProviderImpl versionProvider = mock(VersionProviderImpl.class);
        ConfigurationManager configurationManager = mock(ConfigurationManager.class);
        ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        PostDbSchemaInitializationManager initializationManager =
                new PostDbSchemaInitializationManager(artifactoryHome, configurationManager,
                        versionProvider, null, null, false, null);
        when(versionProvider.getCompoundVersionDetailsFromDbIfExists())
                .thenReturn(new CompoundVersionDetails(ArtifactoryServiceVersionProvider.v711m001.get(),
                        String.valueOf(ArtifactoryServiceVersionProvider.v711m001.get().getRevision()),
                        ClockUtils.epochMillis()));
        when(artifactoryHome.isHaConfigured()).thenReturn(true);

        assertTrue(initializationManager.shouldAcquireInitLock());
    }


    public void testAcquireInitLockInHaNewInstallation() {
        VersionProviderImpl versionProvider = mock(VersionProviderImpl.class);
        ConfigurationManager configurationManager = mock(ConfigurationManager.class);
        ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        PostDbSchemaInitializationManager initializationManager =
                new PostDbSchemaInitializationManager(artifactoryHome, configurationManager,
                        versionProvider, null, null, false, null);
        when(versionProvider.getCompoundVersionDetailsFromDbIfExists())
                .thenReturn(new CompoundVersionDetails(ArtifactoryServiceVersionProvider.v711m001.get(),
                        String.valueOf(ArtifactoryServiceVersionProvider.v711m001.get().getRevision()),
                        ClockUtils.epochMillis()));
        when(artifactoryHome.isHaConfigured()).thenReturn(true);

        assertTrue(initializationManager.shouldAcquireInitLock());
    }

    @Test
    public void testAcquireInitLockInHaWhenVersionIsNull() {
        VersionProviderImpl versionProvider = mock(VersionProviderImpl.class);
        ConfigurationManager configurationManager = mock(ConfigurationManager.class);
        ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        PostDbSchemaInitializationManager initializationManager =
                new PostDbSchemaInitializationManager(artifactoryHome, configurationManager,
                        versionProvider, null, null, true, null);

        when(versionProvider.getCompoundVersionDetailsFromDbIfExists())
                .thenReturn(null);
        when(artifactoryHome.isHaConfigured()).thenReturn(true);

        assertTrue(initializationManager.shouldAcquireInitLock());
    }

    @Test
    public void testNotAcquireInitLockWhenHaIsNotConfiguredAndVersionIsNotCurrent() {
        VersionProviderImpl versionProvider = mock(VersionProviderImpl.class);
        ConfigurationManager configurationManager = mock(ConfigurationManager.class);
        ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        PostDbSchemaInitializationManager initializationManager =
                new PostDbSchemaInitializationManager(artifactoryHome, configurationManager,
                        versionProvider, null, null, false, null);
        when(versionProvider.getCompoundVersionDetailsFromDbIfExists())
                .thenReturn(new CompoundVersionDetails(ArtifactoryServiceVersionProvider.v711m001.get(),
                        String.valueOf(ArtifactoryServiceVersionProvider.v711m001.get().getRevision()),
                        ClockUtils.epochMillis()));
        when(artifactoryHome.isHaConfigured()).thenReturn(false);

        assertFalse(initializationManager.shouldAcquireInitLock());
    }

    @Test
    public void testNotAcquireInitLockInNewInstallationWhenHaIsNotConfigured() {
        VersionProviderImpl versionProvider = mock(VersionProviderImpl.class);
        ConfigurationManager configurationManager = mock(ConfigurationManager.class);
        ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        PostDbSchemaInitializationManager initializationManager =
                new PostDbSchemaInitializationManager(artifactoryHome, configurationManager,
                        versionProvider, null, null, true, null);
        when(versionProvider.getCompoundVersionDetailsFromDbIfExists())
                .thenReturn(null);
        when(artifactoryHome.isHaConfigured()).thenReturn(false);

        assertFalse(initializationManager.shouldAcquireInitLock());
    }

    @Test
    public void testAcquireInitLockWhenVersionIsNullAndHaNotConfigured() {
        VersionProviderImpl versionProvider = mock(VersionProviderImpl.class);
        ConfigurationManager configurationManager = mock(ConfigurationManager.class);
        ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        PostDbSchemaInitializationManager initializationManager =
                new PostDbSchemaInitializationManager(artifactoryHome, configurationManager,
                        versionProvider, null, null, false, null);

        when(versionProvider.getCompoundVersionDetailsFromDbIfExists())
                .thenReturn(null);
        when(artifactoryHome.isHaConfigured()).thenReturn(false);

        assertFalse(initializationManager.shouldAcquireInitLock());
    }

    @Test
    public void testNotAcquireInitLockInHaAndVersionIsCurrent() {
        VersionProviderImpl versionProvider = mock(VersionProviderImpl.class);
        ConfigurationManager configurationManager = mock(ConfigurationManager.class);
        ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        CompoundVersionDetails compoundVersionDetails = mock(CompoundVersionDetails.class);
        PostDbSchemaInitializationManager initializationManager =
                new PostDbSchemaInitializationManager(artifactoryHome, configurationManager,
                        versionProvider, null, null, false, null);

        when(versionProvider.getCompoundVersionDetailsFromDbIfExists())
                .thenReturn(compoundVersionDetails);
        when(compoundVersionDetails.isCurrent())
                .thenReturn(true);
        when(artifactoryHome.isHaConfigured()).thenReturn(true);

        assertFalse(initializationManager.shouldAcquireInitLock());
    }

    @Test
    public void testNotAcquireInitLockWhenHaIsNotConfiguredAndVersionIsCurrent() {
        VersionProviderImpl versionProvider = mock(VersionProviderImpl.class);
        ConfigurationManager configurationManager = mock(ConfigurationManager.class);
        ArtifactoryHome artifactoryHome = mock(ArtifactoryHome.class);
        CompoundVersionDetails compoundVersionDetails = mock(CompoundVersionDetails.class);
        PostDbSchemaInitializationManager initializationManager =
                new PostDbSchemaInitializationManager(artifactoryHome, configurationManager,
                        versionProvider, null, null, false, null);


        when(versionProvider.getCompoundVersionDetailsFromDbIfExists())
                .thenReturn(compoundVersionDetails);
        when(compoundVersionDetails.isCurrent())
                .thenReturn(true);
        when(artifactoryHome.isHaConfigured()).thenReturn(false);

        assertFalse(initializationManager.shouldAcquireInitLock());
    }


}