package org.artifactory.storage.db.search;

import org.apache.commons.lang3.StringUtils;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.aql.AqlConverts;
import org.artifactory.aql.AqlService;
import org.artifactory.aql.api.domain.sensitive.AqlApiItem;
import org.artifactory.aql.api.internal.AqlBase;
import org.artifactory.aql.model.AqlComparatorEnum;
import org.artifactory.aql.model.AqlItemTypeEnum;
import org.artifactory.aql.result.AqlEagerResult;
import org.artifactory.aql.result.rows.AqlItem;
import org.artifactory.aql.util.AqlSearchablePath;
import org.artifactory.common.ConstantValues;
import org.artifactory.fs.FileInfo;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.request.RequestThreadLocal;
import org.artifactory.storage.db.InternalDbService;
import org.jetbrains.annotations.NotNull;
import org.jfrog.common.StreamSupportUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.inject.Inject;
import java.util.List;
import java.util.Set;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import static org.apache.commons.lang3.StringUtils.trim;
import static org.apache.commons.lang3.StringUtils.trimToNull;
import static org.artifactory.aql.api.internal.AqlBase.and;
import static org.artifactory.aql.util.AqlUtils.getSearchClauseForPaths;
import static org.artifactory.aql.util.AqlUtils.getSearchablePathForCurrentFolderAndSubfolders;
import static org.artifactory.mime.DockerNaming.MANIFEST_FILENAME;

/**
 * A utility class to perform several common searches.
 *
 * @author Yossi Shaul
 */
@Service
public class NodeSearcher {
    private static final Logger log = LoggerFactory.getLogger(NodeSearcher.class);
    private final AqlService aqlService;

    @Inject
    public NodeSearcher(AqlService aqlService) {
        this.aqlService = aqlService;
    }

    /**
     * Find files with specific name under the specified repo and (optional) path, including sub directories. This
     * method filters directories and returns only files. <p/>
     * Do not use this method to search only for files directly under the provided paths. Use other methods
     * (such as RepositoryService#getChildrenNames(RepoPath). <p/>
     * None of the input params should contain wildcard characters!
     *
     * @param repo The repository to search under
     * @param path The path to search under - can be empty
     * @param name The name of the files to look for
     * @return Files matching the search criteria
     */
    @Nonnull
    public Set<FileInfo> findFiles(String repo, @Nullable String path, String name) {
        validateInput(repo, name);
        AqlApiItem query = buildFindFilesByNameQuery(trim(repo), trimToNull(path), trim(name));
        AqlEagerResult<AqlItem> results = aqlService.executeQueryEager(query);
        return filterResults(results);
    }

    /**
     * Find files with specific sha256 checksum under the specified repo and (optional) path, including sub directories.
     * None of the input params should contain wildcard characters!
     *
     * @param repo   The repository to search under
     * @param path   The path to search under - can be empty
     * @param sha256 The name of the files to look for
     * @return Files matching the search criteria
     */
    public Set<FileInfo> findFilesBySha256(String repo, @Nullable String path, String sha256) {
        validateInput(repo, sha256);
        AqlApiItem query = buildFindFilesBySha256Query(trim(repo), trimToNull(path), trim(sha256));
        AqlEagerResult<AqlItem> results = aqlService.executeQueryEager(query);
        return filterResults(results);
    }

    public Set<FileInfo> findFilesUnderPath(String repo, @Nullable String path) {
        AqlBase.AndClause<AqlApiItem> rootFilterClause = and();
        List<AqlSearchablePath> sourceFiles =
                getSearchablePathForCurrentFolderAndSubfolders(RepoPathFactory.create(repo, path));
        rootFilterClause.append(getSearchClauseForPaths(sourceFiles));
        rootFilterClause.append(AqlApiItem.type().equal("file"));
        AqlApiItem query = AqlApiItem.create().filter(rootFilterClause);
        return StreamSupportUtils.stream(aqlService.executeQueryEager(query).getResults())
                .map(AqlConverts.toFileInfo)
                .collect(Collectors.toSet());
    }

    private AqlApiItem buildFindFilesByNameQuery(String repo, String path, String name) {
        AqlApiItem query = AqlApiItem.create();
        AqlBase.AndClause<AqlApiItem> andFilter = createRepoAndPathFilters(repo, path);
        andFilter.append(AqlApiItem.name().equal(name));
        return query.filter(andFilter);
    }

    private AqlApiItem buildFindFilesBySha256Query(String repo, String path, String sha256) {
        AqlApiItem query = AqlApiItem.create();
        AqlBase.AndClause<AqlApiItem> andFilter = createRepoAndPathFilters(repo, path);
        if (canFilterBySha256() && isSha256Ready()) {
            andFilter.append(AqlApiItem.sha2().equal(sha256));
        } else {
            andFilter.append(AqlApiItem.property().property("sha256", AqlComparatorEnum.equals, sha256));
        }

        return query.filter(andFilter);
    }

    private boolean canFilterBySha256() {
        return !ConstantValues.dockerPullV1SchemaByDigestEnabled.getBoolean();
    }

    @NotNull
    private AqlBase.AndClause<AqlApiItem> createRepoAndPathFilters(String repo, String path) {
        AqlApiItem.AndClause<AqlApiItem> and = AqlApiItem.and();
        and.append(AqlApiItem.repo().equal(repo));
        if (path != null) {
            AqlBase.CriteriaClause<AqlApiItem> matches = AqlApiItem.path().matches(path + "/*");
            if (ConstantValues.dockerSearchManifestByDigestAndTagEnabled.getBoolean()) {
                AqlBase.CriteriaClause<AqlApiItem> equal = AqlApiItem.path().equal(path);
                AqlApiItem.OrClause<AqlApiItem> or = AqlApiItem.or();
                or.append(matches);
                or.append(equal);
                and.append(or);
            } else {
                and.append(matches);
            }
        }
        // remove the restriction for files only - this is not indexed and it's probably better to filter the results
        and.append(AqlApiItem.type().equal("any"));
        return and;
    }

    private Set<FileInfo> filterResults(AqlEagerResult<AqlItem> results) {
        boolean shouldLogUserInfo = ConstantValues.dockerPullV1SchemaByDigestEnabled.getBoolean();
        Predicate<AqlItem> filter = shouldLogUserInfo ? filesOnlyAndLog() : filesOnly();
        return results.getResults().stream()
                .filter(filter)
                .map(AqlConverts.toFileInfo)
                .collect(Collectors.toSet());
    }

    private Predicate<AqlItem> filesOnly() {
        return i -> i.getType().equals(AqlItemTypeEnum.file);
    }

    private Predicate<AqlItem> filesOnlyAndLog() {
        return i -> {
            boolean isFile = i.getType().equals(AqlItemTypeEnum.file);
            boolean isManifest = isFile && i.getName() != null && i.getName().equals(MANIFEST_FILENAME);
            if (isManifest && !RequestThreadLocal.getRequestPath().endsWith(i.getSha2())) {
                log.debug("Image V1 requested: {} , Client address: {} , User-Agent: {}", i.getPath(),
                        RequestThreadLocal.getClientAddress(), RequestThreadLocal.getUserAgent());
            }
            return isFile;
        };
    }

    private void validateInput(String repo, String name) {
        if (StringUtils.isBlank(repo)) {
            throw new IllegalArgumentException("Repo cannot be cannot be empty");
        }
        if (StringUtils.isBlank(name)) {
            throw new IllegalArgumentException("Name cannot be cannot be empty");
        }
    }

    private boolean isSha256Ready() {
        return ContextHelper.get().beanForType(InternalDbService.class).isSha256Ready();
    }
}
